package dev.shadowsoffire.apotheosis.spawn.modifiers;

import net.minecraft.network.FriendlyByteBuf;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import javax.annotation.Nullable;
import java.util.ArrayList;
import dev.shadowsoffire.apotheosis.Apoth;
import net.minecraft.world.item.crafting.RecipeType;
import net.minecraft.world.item.crafting.RecipeSerializer;
import net.minecraft.core.RegistryAccess;
import net.minecraft.world.level.Level;
import java.util.Iterator;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.spawn.spawner.ApothSpawnerTile;
import java.util.Collection;
import com.google.common.collect.ImmutableList;
import java.util.List;
import net.minecraft.world.item.crafting.Ingredient;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.Container;
import net.minecraft.world.item.crafting.Recipe;

public class SpawnerModifier implements Recipe<Container>
{
    public static final Serializer SERIALIZER;
    protected final ResourceLocation id;
    protected final Ingredient mainHand;
    protected final Ingredient offHand;
    protected final boolean consumesOffhand;
    protected final List<StatModifier<?>> statChanges;
    
    public SpawnerModifier(final ResourceLocation id, final Ingredient mainHand, final Ingredient offHand, final boolean consumesOffhand, final List<StatModifier<?>> stats) {
        this.id = id;
        this.mainHand = mainHand;
        this.offHand = offHand;
        this.consumesOffhand = consumesOffhand;
        this.statChanges = (List<StatModifier<?>>)ImmutableList.copyOf((Collection)stats);
    }
    
    public boolean matches(final ApothSpawnerTile tile, final ItemStack mainhand, final ItemStack offhand) {
        return this.mainHand.test(mainhand) && (this.offHand == Ingredient.f_43901_ || this.offHand.test(offhand));
    }
    
    public boolean apply(final ApothSpawnerTile tile) {
        boolean success = false;
        for (final StatModifier<?> m : this.statChanges) {
            if (m.apply(tile)) {
                success = true;
                tile.m_6596_();
            }
        }
        return success;
    }
    
    public boolean consumesOffhand() {
        return this.consumesOffhand;
    }
    
    public Ingredient getMainhandInput() {
        return this.mainHand;
    }
    
    public Ingredient getOffhandInput() {
        return this.offHand;
    }
    
    public List<StatModifier<?>> getStatModifiers() {
        return this.statChanges;
    }
    
    @Deprecated
    public boolean m_5818_(final Container pContainer, final Level pLevel) {
        return false;
    }
    
    @Deprecated
    public ItemStack m_5874_(final Container pContainer, final RegistryAccess regs) {
        return ItemStack.f_41583_;
    }
    
    @Deprecated
    public boolean m_8004_(final int pWidth, final int pHeight) {
        return false;
    }
    
    @Deprecated
    public ItemStack m_8043_(final RegistryAccess regs) {
        return ItemStack.f_41583_;
    }
    
    public ResourceLocation m_6423_() {
        return this.id;
    }
    
    public RecipeSerializer<?> m_7707_() {
        return (RecipeSerializer<?>)SpawnerModifier.SERIALIZER;
    }
    
    public RecipeType<?> m_6671_() {
        return (RecipeType<?>)Apoth.RecipeTypes.MODIFIER;
    }
    
    @Nullable
    public static SpawnerModifier findMatch(final ApothSpawnerTile tile, final ItemStack mainhand, final ItemStack offhand) {
        final List<SpawnerModifier> recipes = new ArrayList<SpawnerModifier>(tile.m_58904_().m_7465_().m_44013_(Apoth.RecipeTypes.MODIFIER));
        recipes.sort((r1, r2) -> (r1.offHand == Ingredient.f_43901_) ? ((r2.offHand == Ingredient.f_43901_) ? 0 : 1) : -1);
        for (final SpawnerModifier r : recipes) {
            if (r.matches(tile, mainhand, offhand)) {
                return r;
            }
        }
        return null;
    }
    
    static {
        SERIALIZER = new Serializer();
    }
    
    public static class Serializer implements RecipeSerializer<SpawnerModifier>
    {
        public SpawnerModifier fromJson(final ResourceLocation id, final JsonObject obj) {
            final Ingredient mainhand = Ingredient.m_43917_(obj.get("mainhand"));
            final Ingredient offhand = obj.has("offhand") ? Ingredient.m_43917_(obj.get("offhand")) : Ingredient.f_43901_;
            final JsonArray stats = obj.get("stat_changes").getAsJsonArray();
            final List<StatModifier<?>> statChanges = new ArrayList<StatModifier<?>>();
            for (final JsonElement e : stats) {
                statChanges.add(StatModifier.parse(e.getAsJsonObject()));
            }
            return new SpawnerModifier(id, mainhand, offhand, offhand != Ingredient.f_43901_ && obj.get("consumes_offhand").getAsBoolean(), statChanges);
        }
        
        public SpawnerModifier fromNetwork(final ResourceLocation id, final FriendlyByteBuf buf) {
            final Ingredient mainhand = Ingredient.m_43940_(buf);
            final Ingredient offhand = buf.readBoolean() ? Ingredient.m_43940_(buf) : Ingredient.f_43901_;
            final boolean consumesOffhand = buf.readBoolean();
            final List<StatModifier<?>> statChanges = new ArrayList<StatModifier<?>>();
            for (int size = buf.readByte(), i = 0; i < size; ++i) {
                statChanges.add(StatModifier.read(buf));
            }
            return new SpawnerModifier(id, mainhand, offhand, consumesOffhand, statChanges);
        }
        
        public void toNetwork(final FriendlyByteBuf buf, final SpawnerModifier recipe) {
            recipe.mainHand.m_43923_(buf);
            buf.writeBoolean(recipe.offHand != Ingredient.f_43901_);
            if (recipe.offHand != Ingredient.f_43901_) {
                recipe.offHand.m_43923_(buf);
            }
            buf.writeBoolean(recipe.consumesOffhand);
            buf.writeByte(recipe.statChanges.size());
            recipe.statChanges.forEach(m -> m.write(buf));
        }
    }
}
